import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { CellMeasurer, CellMeasurerCache } from 'react-virtualized';
// helpers
import { columnToLetter, coordToString } from '../helpers';
// components
import ExcelGrid from './ExcelGrid';
import CellBody from './CellBody';


class Sheet extends Component {
  static propTypes = {
    onChange: PropTypes.func.isRequired,
    inputs: PropTypes.instanceOf(Map).isRequired,
    calculated: PropTypes.instanceOf(Map).isRequired,
    height: PropTypes.number.isRequired,
    width: PropTypes.number.isRequired,
    cellWidth: PropTypes.number.isRequired,
    cellHeight: PropTypes.number.isRequired,
    columnCount: PropTypes.number.isRequired,
    rowCount: PropTypes.number.isRequired,
  }

  constructor(props) {
    super(props);
    this.cache = new CellMeasurerCache({
      defaultWidth: props.cellWidth,
      defaultHeight: props.cellHeight,
      minWidth: props.cellWidth,
      minHeight: props.cellHeight,
    });
  }

  cellRendererBody = ({
    columnIndex,
    isScrolling,
    key,
    rowIndex,
    style,
    parent,
  }) => {
    const coord = coordToString({ column: columnIndex, row: rowIndex });
    const {
      onChange,
      inputs,
      calculated,
    } = this.props;

    const style1 = Object.assign({}, style);

    if (rowIndex % 2 === 0) {
      style1.backgroundColor = 'white';
    } else {
      style1.backgroundColor = '#f0f0f0';
    }

    style1.cursor = 'cell';
    let inputValue = '';
    let calculatedValue = '';

    if (inputs.has(coord)) {
      inputValue = inputs.get(coord);
    }

    if (calculated.has(coord)) {
      calculatedValue = calculated.get(coord);
    } else if (inputValue !== '') {
      style1.backgroundColor = 'rgba(255, 0, 0, 0.1)';
      calculatedValue = inputValue;
    }

    let content = isScrolling
      ? <div key={key} style={style1}>...</div>
      : (
        <CellBody
          key={key}
          style={style1}
          onChange={(e) => {
            onChange(coord, e);
            parent.recomputeGridSize({ columnIndex, rowIndex });
          }}
          inputValue={inputValue}
          calculatedValue={calculatedValue}
        />
      );
    const mouse = {
      startX: 0, endX: 0, startY: 0, endY: 0,
    };

    if (rowIndex === 0) {
      content = (
        <div
          key={key}
          draggable
          style={style1}

        >
          <div style={
              {
                display: 'flex',
                flexDirection: 'row',
                height: '100%',
              }
            }
          >
            <div style={
              {
                textAlign: 'center',
                width: '100%',
              }
            }
            >
              {columnToLetter(columnIndex)}
            </div>
            <div
              style={{
                cursor: 'col-resize',
                backgroundColor: 'grey',
                width: '4px',
                height: '100%',
              }}
              draggable
              onDragStart={(e) => {
                mouse.startX = e.screenX;
                e.dataTransfer.setData('text/plain', 'draggable');
              }
              }
              onDragEnd={(e) => {
                mouse.endX = e.screenX;
                this.cache.set(rowIndex, columnIndex, style.width - mouse.startX + mouse.endX, this.cache.defaultHeight);
                parent.recomputeGridSize({ columnIndex, rowIndex });
              }}
            />
          </div>
        </div>
      );
    }


    if (columnIndex === 0) {
      content = rowIndex === 0
        ? <div key={key} style={style1} />
        : (
          <div
            key={key}
            style={style1}
          >
            <div style={
              {
                display: 'flex',
                alignContent: 'space-between',
                flexDirection: 'column',
                height: '100%',
              }
            }
            >
              <div style={
              {
                textAlign: 'center',
                height: '100%',
              }
            }
              >
                {rowIndex}
              </div>
              <div
                style={{
                  cursor: 'row-resize',
                  backgroundColor: 'grey',
                  height: '4px',
                  width: '100%',
                }}
                draggable
                onDragStart={(e) => {
                  mouse.startY = e.screenY;
                  e.dataTransfer.setData('text/plain', 'draggable');
                }
        }
                onDragEnd={(e) => {
                  mouse.endY = e.screenY;
                  if (Math.abs(mouse.startY - mouse.endY) >= this.cache.defaultHeight) {
                    this.cache.set(rowIndex, columnIndex, this.cache.defaultHeight, style.height - mouse.startY + mouse.endY);
                    parent.recomputeGridSize({ columnIndex, rowIndex });
                  }
                }}
              />
            </div>
          </div>
        );
    }

    return (
      <CellMeasurer
        cache={this.cache}
        columnIndex={columnIndex}
        key={key}
        parent={parent}
        rowIndex={rowIndex}
      >
        {content}
      </CellMeasurer>);
  }

  render() {
    const {
      height,
      width,
      rowCount,
      columnCount,
    } = this.props;
    return (
      <ExcelGrid
        cellRendererTop={this.cellRendererTop}
        height={height}
        width={width}
        deferredMeasurementCache={this.cache}
        cellWidth={this.cache.columnWidth}
        columnCount={columnCount}
        cellHeight={this.cache.rowHeight}
        rowCount={rowCount}
        cellRendererLeft={this.cellRendererLeft}
        cellRendererBody={this.cellRendererBody}
      />
    );
  }
}

export default Sheet;
